#!/usr/bin/env python
#-*- coding:utf-8 -*-

#%Module
#% description: Produce low-resolution grids to assist plotting Pareto boundaries
#% keywords: raster, vector
#% keywords: pareto boundary, accuracy assessment, low resolution, dichotomic, thematic maps
#%end

#%option
#% key: highres
#% type: double
#% description: Pixel (high) resolution of reference map
#% answer: 30
#% required: yes
#%end

#%option
#% key: lowres
#% type: double
#% description: Pixel (low) resolution of map to be assessed
#% answer: 500
#% required: yes
#%end

#%option
#% key: thresholds_int
#% type: integer
#% key_desc: threshold values
#% description: Threshold values for Pareto optimal maps
#% multiple: yes
#% answer: 1, 10, 20, 30, 40, 50, 60, 70, 80, 90, 100
#% required: yes
#%end


# import modules
# import sys
import grass.script as grass





# main function ##############################################################

def main():
    """
    Step ?.?: Preparative actions for low resolution vector grid creation
    """


#############################################################################
### input: vector maps

    # reference_coi_points
    reference_coi_points_raw = grass.read_command('g.mlist',\
                                            type = "vect",\
                                            pat = "pareto_ref_coi_points___*")
    reference_coi_points = reference_coi_points_raw.splitlines()


    # vectorised classification maps
    classification_vectors_raw = grass.read_command('g.mlist',\
                                                    type = "vect",\
                                                    pat = "pareto_classification_V___*")
    classification_vectors = classification_vectors_raw.splitlines()


    # lowres = options['lowres']
    lowres = 500

### output:
#############################################################################


    print """\n > Creating low resolution vector grid(s)"""


    # loop over vectorised classification maps ###############################
    for cV in classification_vectors:
        print """\n     (Running process for '%s')""" % cV
    # loop over vectorised classification maps ###############################


        # input: ?

        # loop over reference CoI points
        for RCoIP in reference_coi_points:
            """
            select classified CoIs within region that correspond to reference CoIs
            """


            print """\n   * Selecting Class of Interest from:\n      '%s' \n     that corresponds to:\n      '%s'""" % (cV, RCoIP)


            # match region carefully to get a low-resolution grid with res = lowres
            grass.run_command('g.region',\
            flags = "a",\
            res = lowres,\
            vect = RCoIP,)
            print "\n     + Region matched to '%s'" % RCoIP


            # v.in.region
            tmpregion_RCoIP = "region_of_" + RCoIP

            grass.run_command('v.in.region',\
            flags = "-o",\
            quiet = True,\
            out = tmpregion_RCoIP)
            #print "     * Temp region vector created for to '%s'" % RCoIP


            # extract classified CoI that correspond to reference CoI

            # classification_vector_coi
            classification_vector_coi = cV.replace("V", "V_coi") + "__FOR__" + RCoIP.replace("pareto_", "")

            grass.run_command('v.select',\
            flags = "-o",\
            quiet = True,\
            ainput = cV,\
            binput = tmpregion_RCoIP,\
            operator = "overlap",\
            output = classification_vector_coi)
            print """\n     + Class of Interest area(s)\n      '%s'\n     extracted from\n      '%s'""" % (classification_vector_coi, cV)



            # remove temp region
            grass.run_command('g.remove',\
            quiet = True,\
            vect = tmpregion_RCoIP)


            """
            Step ?.?: Create low resolution vector grid over reference Class of Interest maps
            """

            print """\n   * Creating low resolution vector grid over reference Class of Interest maps"""

##############################################################################
            ### input: Class of Interest vector points, classified Class of Interest vector polygons

### This name is very important. Make it smart!
            lowres_vector_grid = RCoIP.replace("pareto", "pareto_grid___OVER__") + "___ON___" + cV.replace("pareto_", "")


            ### intermediate: number of rows and cols of region
            ### output: low resolution grid
##############################################################################

#        for RCoIP in (reference_coi_points, ):



            # what if (classification_vector_coi)['boundaries'] == 0 ?
            num_bound = grass.vector_info_topo(map = classification_vector_coi)['boundaries']

            if num_bound != 0:

                # match region carefully with res= lowres
                grass.run_command('g.region',\
                flags = "a",\
                quiet = True,\
                vect = RCoIP + "," + classification_vector_coi,\
                res = lowres)
                print "\n     + Region carefully matched to fit\n      '%s'\n     and\n      '%s'"\
                 % (RCoIP, classification_vector_coi)

            else:

                # match region carefully with res= lowres
                grass.run_command('g.region',\
                flags = "a",\
                quiet = True,\
                vect = RCoIP,\
                res = lowres)
                print "\n     + Region carefully matched to fit\n      '%s'\n     and\n      '%s'"\
                 % (RCoIP, classification_vector_coi)


            # get number of rows, cols of region
            rows = int(grass.region()['rows'])
            cols = int(grass.region()['cols'])
            cells = rows * cols
        #   print 'rows=%d \ncols=%d' % (rows, cols)


            # create low resolution vector grid

# find a better naming convention !

            grass.run_command('v.mkgrid',\
            flags = "-o",\
            quiet = True,\
            map = lowres_vector_grid,\
            grid = '%d,%d' % (rows,cols),\
            position = "region")

            print """\n     + Vector grid of size %d rows x %d ( %s cells) columns created"""\
            % (rows, cols, cells)

            print """\n   Maps created:
                 %s
                 %s""" % (lowres_vector_grid, classification_vector_coi)


    print "\n >>> Raster material successfully vectorised"


    # end of loop over vectorised classification maps ########################


#  make this script both importable and runnable
if __name__ == "__main__":
    options, flags = grass.parser()
    main()

